/*
 *  linux/include/asm-arm/arch-psionw/io.h
 *
 *  Copyright (C) 1999 ARM Limited
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
 */

#include <asm/hardware/psionw.h>
#include <asm/delay.h>

#ifndef __ASM_ARM_ARCH_IO_H
#define __ASM_ARM_ARCH_IO_H

#define IO_SPACE_LIMIT 0xffffffff

#ifdef __io
#undef __io
#endif

#define __mem_pci(a)		((unsigned long)(a))
#define __mem_isa(a)		((unsigned long)(a))

/*
 * Generic virtual read/write
 */
#define __arch_getw(a)		(*(volatile unsigned short *)(a))
#define __arch_putw(v,a)	(*(volatile unsigned short *)(a) = (v))

#define etna_readb(p)		(*(volatile u8 *)(ETNA_V_BASE + (p)))
#define etna_writeb(v,p)	(*(volatile u8 *)(ETNA_V_BASE + (p)) = (v))

#define etna_readl(p)		(*(volatile unsigned int *)(ETNA_V_BASE + (p)))

/*
 * Epoc boot EEPROM reading. Writing would be dangerous, as there is
 * no known alternate boot method.
 */
#define psion_br_read(p)	(*(volatile u8 *)(PSION_V_BR + (p)))

/*
 * Epoc boot flash reading. Writing would be dangerous, as there is
 * no known alternate boot method.
 */
#define psion_bf_read(p)	(*(volatile u8 *)(PSION_V_BF + (p)))

#define DEBUG_MEM_BASE()	({ printk(" 0x%02x-0x%02x", (*(volatile u8 *)(CF1_V_BASE + CF_MEM_BASE + 0x2)), \
				(*(volatile u8 *)(CF1_V_BASE + CF_MEM_BASE + 0x7)));  })

#define ETNA_GET_MEMBASE(p)	(*(volatile u8 *)(CF1_V_BASE + CF_MEM_BASE + (p)))

/*
 * Reads the CompactFlash ready status. Defined at least for SanDisk cards.
 * 0x1f7 clears the pending interrupt, while 0x3f6 does not.
 */
#define CF_GET_STAT()		(*(volatile u8 *)(CF1_V_BASE + CF_IO8_BASE + 0x1f7))
#define CF_GET_ALTSTAT()	(*(volatile u8 *)(CF1_V_BASE + CF_IO8_BASE + 0x3f6))
#define CF_CARD_BUSY_BIT	(1 << 7)

#define etna_mem_inb(p)		({ unsigned int __v =__raw_readb(CF1_V_BASE + CF_MEM_BASE + (p)); __v; })

#define etna_mem_outb(v,p)	__raw_writeb(v, CF1_V_BASE + CF_MEM_BASE + (p))

/*
 * Most CF cards need an extra delay before a 16-bit write.
 * Otherwise we may get tons of ide status=0x58 errors and a
 * corrupted disk. The delay time needed depends on the card
 * and needs to be somewhere between 300 and 1500.
 *
 * The problem happens in the following way:
 * 
 * 1. Ide driver issues a write command to the 8-bit command bus
 * 2. CompactFlash card sets DRQ etc
 * 3. Ide driver waits for not busy and checks for DRQ etc
 * 4. Ide driver starts to write the data to the 16-bit bus
 * 5. Ide driver checks for a successful write
 * 6. CompactFlash card still expects more data, and gives status 0x58
 * 7. Ide driver produces an error, as it expects 0x50 but gets 0x58
 *
 * So it looks like the Psion 32-bit to 16-bit translation bus is not
 * ready to accept data, and the beginning of the write never makese it
 * to the CompactFlash.
 */
static __inline__ void
etna_status_check(void)
{

#if 0
	psionw_writeb(psionw_readb(PBDDR) | 0x3, PBDDR);
	psionw_writeb(psionw_readb(PBDR) & ~0x3, PBDR);

	psionw_writeb(psionw_readb(PBDDR) & ~0x3, PBDDR);
	while ((psionw_readb(PBDR) & 0x3) != 0x3) {
		printk(".");
		udelay(1);
	}
	printk("o");
#endif
	udelay(1500);
}


/* Boundary alignment for ETNA CF access, not currently used */
#define PCMCIA_IO16(a) (CF1_V_BASE + 0x0c000000 + ((a) & ~3) + (((a) & 2) << 24) )

#ifdef DEBUG_INSW
#define INSW_DEBUG(x...)	printk(x)
#else
#define INSW_DEBUG(x...)	do { ; } while(0)
#endif	

/*
 * Translate a series of N x 16-bit reads into N x 32-bit reads
 *
 * Used for ide flash cards only. The automatic translation does not work for
 * network cards, for example. See also etna_insw_direct.
 */
static __inline__ void
etna_insw_ide(unsigned int port, void *to, int len)
{
	int i;

	for (i = 0; i < len; i++) {
		((unsigned short *) to)[i] = (unsigned short) __raw_readl(CF1_V_BASE + CF_IO16_BASE + port);
		//((unsigned short *) to)[i] = (unsigned short) (__raw_readl(PCMCIA_IO16(port)) & 0xffff);
	}
}

/*
 * Translate a series of N x 16-bit reads into N/2 x 32-bit reads for ETNA.
 *
 * Used for non-ide cards.
 */
static __inline__ void
etna_insw_direct(unsigned int port, void *to, int len)
{
	int i, val = 0;

	INSW_DEBUG("\nI <--- Reading %d half words from port 0x%04x\n", len, port);
	for (i = 0; i < len; i++) {
		if (i & 0x1) {
			val = __raw_readl(CF1_V_BASE + CF_IO16_BASE + port);
			INSW_DEBUG("  +<-- 0x%08x\n", val);
			((unsigned short *) to)[i - 1] = (unsigned short) (val & 0xffff);
			((unsigned short *) to)[i] = (unsigned short) (val >> 16);
			INSW_DEBUG("	 +<-- flush to[%d] = 0x%04x  to[%d] = 0x%04x\n", i - 1, 
				   ((unsigned short *) to)[i - 1], i,
				   ((unsigned short *) to)[i]);
		}
	}

	if (len & 0x1) {
		val = __raw_readl(CF1_V_BASE + CF_IO16_BASE + port);
		((unsigned short *) to)[len - 1] = (unsigned short) (val & 0xffff);
		INSW_DEBUG("	 +<-- flush to[%d] = 0x%04x\n", len - 1, 
			   ((unsigned short *) to)[i - 1]);
	}
}


#ifdef DEBUG_OUTSW
#define OUTSW_DEBUG(x...)	printk(x)
#else
#define OUTSW_DEBUG(x...)	do { ; } while(0)
#endif	

/*
 * Capures selected IO port values for debugging
 */
static __inline__ void
psion_debug_regb(char *res, char* desc, unsigned int port, int range)
{
	int len, i;

	if (res == NULL)
		return;

	len = strlen(res);

	if (len > 0)
		res += len;

	sprintf(res, "%s 0x%08x: ", desc, port);
	res += strlen(desc) + 13;

	for (i = 0; i < range; i++) {
		sprintf(res, "0x%02x ", __raw_readb(port + i));
		res += 5;
	}
	//sprintf(res, "\n");
	*res  = '\n';
	res++;
	//sprintf(res, "\0");
	*res = '\0';
	res++;
}


/*
 * Translate a series of N x 16-bit writes into N x 32-bit writes
 *
 * Used for ide flash cards only. The automatic translation does not work for
 * network cards, for example. See also etna_outsw_direct.
 */
static __inline__ void
etna_outsw_ide(unsigned int port, const void *from, int len)
{
	int i;

	etna_status_check();

       	for (i = 0; i < len; i++) {
		__raw_writel(((unsigned short *) from)[i], CF1_V_BASE + CF_IO16_BASE + port);
		//__raw_writel(((unsigned short *) from)[i], PCMCIA_IO16(port));
	}
}

/*
 * Translate a series of N x 16-bit writes into N/2 x 32-bit writes for ETNA.
 *
 * Used for non-ide cards.
 */
static __inline__ void
etna_outsw_direct(unsigned int port, const void *from, int len)
{
	int i, val, prev = 0;

	OUTSW_DEBUG("\nO ---> Writing %d half words to port 0x%04x\n", len, port);

	for (i = 0; i < len; i++) {
		val = (unsigned int) ( ((unsigned short *) from)[i] );

		OUTSW_DEBUG("  +--> 0x%04x\n", val);

		if (i & 0x1) {
			__raw_writel( ((val<<16) | prev), CF1_V_BASE+CF_IO16_BASE+port);
			OUTSW_DEBUG("	  +--> flush 0x%08x\n", ((val<<16) | prev));
		}
		prev = val;
	}
	if (len & 0x1) {
		__raw_writel( (val), CF1_V_BASE+CF_IO16_BASE+port);
		OUTSW_DEBUG("	  +--> flush 0x%08x\n", (val));
	}
}

#define outb(v,p)		__raw_writeb(v, CF1_V_BASE + CF_IO8_BASE + (p))
#define outw(v,p)		__raw_writel(v, CF1_V_BASE + CF_IO16_BASE + (p))
#define outl(v,p)		printk("Error: Unsupported outl called 0x%x\n", (p))

#define inb(p)			({ unsigned int __v =__raw_readb(CF1_V_BASE + CF_IO8_BASE + (p)); __v; })
#define inw(p)			({ unsigned int __v =__raw_readl(CF1_V_BASE + CF_IO16_BASE + (p)) & 0xffff; __v; })
#define inl(p)			printk("Error: Unsupported inl called 0x%x\n", (p))

#define outsb(p,d,l)		__raw_writesb(CF1_V_BASE + CF_IO8_BASE + p,d,l)
#define outsw(p,d,l)		etna_outsw_direct(p,d,l)
#define outsl(p,d,l)		printk("Error: Unsupported outsl called 0x%x\n", (p))

#define insb(p,d,l)		__raw_readsb(CF1_V_BASE + CF_IO8_BASE + p,d,l)
#define insw(p,d,l)		etna_insw_direct(p,d,l)
#define insl(p,d,l)		printk("Error: Unsupported insl called 0x%x\n", (p))

/*
 * ioremap support - validate a PCI memory address, and convert it
 * to a physical address for the page tables.
 */
#define iomem_valid_addr(iomem,size)	(1)
#define iomem_to_phys(iomem)		(iomem)

#endif
